#include "defines.h"
#include "interrupt.h"
#include "serial.h"
#include "lib.h"

void trush()
{
  while (tstc())
    getc();
}

char *sendcmd(char *cmd)
{
  static char buf[32];
  puts(cmd);
  puts("\n");
  do {
    gets(buf);
  } while (buf[0] == '\0');
  return buf;
}

int get_ls_come() { return !strcmp(sendcmd("LS_COME"), "ON") ? 1 : 0; }
int get_ls_pass() { return !strcmp(sendcmd("LS_PASS"), "ON") ? 1 : 0; }
int get_rs_come() { return !strcmp(sendcmd("RS_COME"), "ON") ? 1 : 0; }
int get_rs_pass() { return !strcmp(sendcmd("RS_PASS"), "ON") ? 1 : 0; }
int get_us_push() { return !strcmp(sendcmd("US_PUSH"), "ON") ? 1 : 0; }
int get_ds_push() { return !strcmp(sendcmd("DS_PUSH"), "ON") ? 1 : 0; }

int passing;

void set_passing()
{
  static int left_passing;
  static int right_passing;
  if (get_ls_come()) left_passing = 1;
  if (get_ls_pass()) left_passing = 0;
  if (get_rs_come()) right_passing = 1;
  if (get_rs_pass()) right_passing = 0;
  passing = left_passing || right_passing;
}

void light()
{
  static int i = 0;
  i = (i + 1) % 4;
  if (i == 0) sendcmd("LIGHTON");
  if (i == 2) sendcmd("LIGHTOFF");
}

enum {
  STATUS_OPEN,
  STATUS_CLOSE,
  STATUS_OPENING,
  STATUS_CLOSING,
};

int status_open()
{
  if (passing) {
    sendcmd("CLOSE");
    return STATUS_CLOSING;
  }
  return STATUS_OPEN;
}

int status_close()
{
  if (!passing) {
    sendcmd("OPEN");
    return STATUS_OPENING;
  }
  return STATUS_CLOSE;
}

int status_opening()
{
  if (get_us_push()) {
    sendcmd("STOP");
    return STATUS_OPEN;
  }
  if (passing) {
    sendcmd("CLOSE");
    return STATUS_CLOSING;
  }
  return STATUS_OPENING;
}

int status_closing()
{
  if (get_ds_push()) {
    sendcmd("STOP");
    return STATUS_CLOSE;
  }
  return STATUS_CLOSING;
}

void mainloop()
{
  int status = 0;

  while (1) {
    set_passing();
    switch (status) {
    case STATUS_OPEN:    status = status_open();    break;
    case STATUS_CLOSE:   status = status_close();   break;
    case STATUS_OPENING: status = status_opening(); break;
    case STATUS_CLOSING: status = status_closing(); break;
    }
    if (passing)
      light();
  }
}

int main(void)
{
  INTR_DISABLE; /* ̵ˤ */

#ifdef SIMULATOR
  extern int bss_start, ebss;

  memset(&bss_start, 0, (long)&ebss - (long)&bss_start);

  /* եȥߥ٥ */
  softvec_init();

  /* ꥢν */
  serial_init(SERIAL_DEFAULT_DEVICE);
#endif

#if 0
  puts("kozos boot succeed!\n");
#endif

  mainloop();

  /* ˤäƤʤ */

  return 0;
}
